"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var TransferService_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.TransferService = void 0;
const common_1 = require("@nestjs/common");
const origin_247_certificate_1 = require("@energyweb/origin-247-certificate");
const EnergyTransferRequest_repository_1 = require("./repositories/EnergyTransferRequest.repository");
const cqrs_1 = require("@nestjs/cqrs");
const GetTransferSites_query_1 = require("./queries/GetTransferSites.query");
const common_2 = require("@nestjs/common");
const ValidateTransferCommand_1 = require("./commands/ValidateTransferCommand");
const EnergyTransferRequest_1 = require("./EnergyTransferRequest");
const events_1 = require("./events");
let TransferService = TransferService_1 = class TransferService {
    constructor(certificateService, energyTransferRequestRepository, validateCommands, queryBus, commandBus, eventBus) {
        this.certificateService = certificateService;
        this.energyTransferRequestRepository = energyTransferRequestRepository;
        this.validateCommands = validateCommands;
        this.queryBus = queryBus;
        this.commandBus = commandBus;
        this.eventBus = eventBus;
        this.logger = new common_2.Logger(TransferService_1.name);
    }
    async issue(command) {
        const { generatorId, energyValue, transferDate, fromTime, toTime, metadata } = command;
        const sites = await this.queryBus.execute(new GetTransferSites_query_1.GetTransferSitesQuery({ generatorId }));
        if (!sites) {
            this.logger.log(`No sites queries for ${generatorId}`);
            return;
        }
        const request = await this.energyTransferRequestRepository.createNew({
            buyerAddress: sites.buyerAddress,
            sellerAddress: sites.sellerAddress,
            volume: energyValue,
            generatorId,
            transferDate
        });
        const certificate = await this.certificateService.issue({
            deviceId: generatorId,
            energyValue: energyValue,
            fromTime,
            toTime,
            userId: sites.sellerAddress,
            toAddress: sites.sellerAddress,
            metadata
        });
        request.updateCertificateId(certificate.id);
        await this.energyTransferRequestRepository.save(request);
        const isCertificatePersisted = Boolean(await this.certificateService.getById(certificate.id));
        if (isCertificatePersisted) {
            await this.persistRequestCertificate(certificate.id);
        }
    }
    async persistRequestCertificate(certificateId) {
        const request = await this.energyTransferRequestRepository.findByCertificateId(certificateId);
        if (!request) {
            this.logger.warn(`
                No transfer request found for certificate: ${certificateId}.
                This can mean, that there was a race condition, and CertificatePersisted event was received,
                before we could save the certificate id on ETR.
            `);
            return;
        }
        request.markCertificatePersisted();
        await this.energyTransferRequestRepository.save(request);
        await this.startValidation(request);
    }
    async startValidation(request) {
        request.startValidation(this.validateCommands.map((c) => c.name));
        await this.energyTransferRequestRepository.save(request);
        if (request.isValid()) {
            this.eventBus.publish(new events_1.ValidatedTransferRequestEvent({
                requestId: request.id
            }));
            return;
        }
        await Promise.all(this.validateCommands.map(async (Command) => {
            const result = await this.executeCommand(Command, request);
            await this.updateValidationStatus({
                validatorName: Command.name,
                requestId: request.id,
                status: result.validationResult
            });
        }));
    }
    async updateValidationStatus(command) {
        const { requestId, validatorName: commandName, status } = command;
        await this.energyTransferRequestRepository.updateWithLock(requestId, (request) => {
            request.updateValidationStatus(commandName, status);
        });
        const request = await this.energyTransferRequestRepository.findById(requestId);
        if (request && request.isValid()) {
            this.eventBus.publish(new events_1.ValidatedTransferRequestEvent({
                requestId: request.id
            }));
        }
    }
    async transferCertificate(requestId) {
        const request = await this.energyTransferRequestRepository.findById(requestId);
        if (!request) {
            this.logger.error(`Received transfer request for request ${requestId}, but not such found`);
            return;
        }
        if (!request.certificateId) {
            this.logger.error(`Received transfer request for request ${requestId}, but this request has no certificate`);
            return;
        }
        if (!request.isValid()) {
            this.logger.error(`Received transfer request for request ${requestId}, but this request is not valid`);
            return;
        }
        await this.certificateService.transfer({
            certificateId: request.certificateId,
            fromAddress: request.sites.sellerAddress,
            toAddress: request.sites.buyerAddress,
            energyValue: request.volume
        });
    }
    async executeCommand(Command, request) {
        try {
            const command = new Command(request.toPublicAttrs());
            return await this.commandBus.execute(command);
        }
        catch (e) {
            this.logger.error(`
                One of validation commands (${Command.name}) returned error for request: ${request.id}:
                "${e.message}"
            `);
            return { validationResult: EnergyTransferRequest_1.TransferValidationStatus.Error };
        }
    }
};
TransferService = TransferService_1 = __decorate([
    common_1.Injectable(),
    __param(0, common_1.Inject(origin_247_certificate_1.CERTIFICATE_SERVICE_TOKEN)),
    __param(1, common_1.Inject(EnergyTransferRequest_repository_1.ENERGY_TRANSFER_REQUEST_REPOSITORY)),
    __param(2, common_1.Inject(ValidateTransferCommand_1.VALIDATE_TRANSFER_COMMANDS_TOKEN)),
    __metadata("design:paramtypes", [origin_247_certificate_1.CertificateService, Object, Array, cqrs_1.QueryBus,
        cqrs_1.CommandBus,
        cqrs_1.EventBus])
], TransferService);
exports.TransferService = TransferService;
//# sourceMappingURL=transfer.service.js.map